#include <Windows.h>
#include <commdlg.h>
#include "../pluginsdk/_plugins.h"
#include "../pluginsdk/bridgemain.h"
#include "../pluginsdk/_scriptapi.h"
#include "../pluginsdk/_scriptapi_module.h"
#include "../pluginsdk/_scriptapi_debug.h"
#include "../pluginsdk/_scriptapi_gui.h"
#include "plugin.h"
#include "scenario_scripts.h"
#include <string>

// Global variables
int g_hPlugin = 0;
static int g_hMenu = 0;
static int g_hMenuAutoImport = 0;
static bool g_autoImportEnabled = false;

// Fix the file dialog function
bool ShowFileOpenDialog(const char* title, const char* defaultDir, char* selectedFile, const char* filter)
{
    // Use the Windows API to show a file dialog
    OPENFILENAMEA ofn;
    ZeroMemory(&ofn, sizeof(ofn));
    ofn.lStructSize = sizeof(ofn);
    ofn.hwndOwner = GuiGetWindowHandle();
    ofn.lpstrFile = selectedFile;
    ofn.nMaxFile = MAX_PATH;
    ofn.lpstrFilter = filter;
    ofn.nFilterIndex = 1;
    ofn.lpstrInitialDir = defaultDir;
    ofn.lpstrTitle = title;
    ofn.Flags = OFN_PATHMUSTEXIST | OFN_FILEMUSTEXIST;

    return GetOpenFileNameA(&ofn) != 0;
}

// Fix the input box function
bool ShowInputBox(const char* title, const char* text, char* result)
{
    return GuiGetLineWindow(title, result);
}

// Also add a function to get the selection
bool GetSelection(duint* start, duint* end)
{
    SELECTIONDATA sel;
    if (GuiSelectionGet(GUI_DISASSEMBLY, &sel))
    {
        *start = sel.start;
        *end = sel.end;
        return true;
    }
    return false;
}

// Plugin initialization
bool pluginit(PLUG_INITSTRUCT *initStruct)
{
  initStruct->pluginVersion = PLUGIN_VERSION;
  initStruct->sdkVersion = PLUG_SDKVERSION;
  strncpy_s(initStruct->pluginName, PLUGIN_NAME, _TRUNCATE);

  // Save plugin handle
  g_hPlugin = initStruct->pluginHandle;

  // Register callbacks
  _plugin_registercallback(g_hPlugin, CB_MENUENTRY, (CBPLUGIN)CBMENUENTRY);

  // Create global scenario manager
  g_scenarioManager = new ScenarioManager();
  if (!g_scenarioManager->Initialize())
  {
    _plugin_logprintf("[Scenario Scripts] Failed to initialize scenario manager\n");
    return false;
  }

  // Sync settings state
  g_autoImportEnabled = g_scenarioManager->GetAutoImportEnabled();

  return true;
}

// Plugin cleanup
void plugstop()
{
  // Release resources
  if (g_scenarioManager)
  {
    delete g_scenarioManager;
    g_scenarioManager = nullptr;
  }
}

// Plugin setup
void plugsetup(PLUG_SETUPSTRUCT *setupStruct)
{
  // Add menu item to main menu
  g_hMenu = setupStruct->hMenu;

  // Create submenu
  int hMenuScenarios = _plugin_menuadd(g_hMenu, "&Scenario Scripts");

  // File operations
  _plugin_menuaddentry(hMenuScenarios, MENU_LOAD_SCRIPT, "📂 &Load Script/Structure Database");
  _plugin_menuaddentry(hMenuScenarios, MENU_CLEAR, "🗑️ &Clear Data");
  _plugin_menuaddseparator(hMenuScenarios);

  // Structure operations
  _plugin_menuaddentry(hMenuScenarios, MENU_VIEW_STRUCTURE, "👁️ &View Structure");
  _plugin_menuaddentry(hMenuScenarios, MENU_LIST_STRUCTURES, "📋 &List All Structures");
  _plugin_menuaddseparator(hMenuScenarios);

  // Analysis
  g_hMenuAutoImport = _plugin_menuaddentry(hMenuScenarios, MENU_AUTO_IMPORT, "🔍 &Auto Import Analysis");
  _plugin_menuaddseparator(hMenuScenarios);

  // Help
  _plugin_menuaddentry(hMenuScenarios, MENU_HELP, "❓ &Help");

  // Set menu state according to loaded settings
  _plugin_menuentrysetchecked(g_hMenu, MENU_AUTO_IMPORT, g_autoImportEnabled);

  _plugin_logprintf("[Scenario Scripts] Plugin loaded successfully!\n");
  _plugin_logprintf("[Scenario Scripts] Use the 'Plugins -> Scenario Scripts' menu to manage scripts and structures.\n");
}

// Menu callback
void CBMENUENTRY(CBTYPE cbType, PLUG_CB_MENUENTRY *info)
{
  switch (info->hEntry)
  {
  case MENU_LOAD_SCRIPT:
  {
    char filePath[MAX_PATH] = "";
    char currentDir[MAX_PATH] = "";
    GetCurrentDirectoryA(MAX_PATH, currentDir);

    if (strlen(currentDir) > 0)
    {
      if (g_scenarioManager)
      {
        char fileName[MAX_PATH] = "";
        if (ShowFileOpenDialog("Open Script or Structure Database", currentDir, fileName, "Text Files (*.txt)\0*.txt\0All Files (*.*)\0*.*\0"))
        {
          std::string path = fileName;
          if (path.find(".txt") != std::string::npos)
          {
            // Try to load as structure database first
            if (g_scenarioManager->LoadStructureDatabase(path))
            {
              _plugin_logprintf("[Scenario Scripts] Structure database loaded: %s\n", path.c_str());
            }
            // Then try as script
            else if (g_scenarioManager->LoadScriptFromFile(path))
            {
              _plugin_logprintf("[Scenario Scripts] Script loaded: %s\n", path.c_str());
            }
            else
            {
              _plugin_logprintf("[Scenario Scripts] Failed to load file: %s\n", path.c_str());
            }
          }
        }
      }
    }
    break;
  }

  case MENU_VIEW_STRUCTURE:
  {
    if (g_scenarioManager)
    {
      char structName[256] = "";
      duint address = 0;

      // Special handling for known structures
      bool useSelection = true;

      // First ask for structure name
      if (ShowInputBox("View Structure", "Enter structure name:", structName))
      {
        // Check if it's a special structure like PEB
        std::string structNameStr = structName;
        if (structNameStr == "PEB")
        {
          // Get PEB address
          bool success = false;
          duint pebAddress = DbgEval("peb", &success);
          if (success && pebAddress != 0)
          {
            address = pebAddress;
            useSelection = false;
            _plugin_logprintf("[Scenario Scripts] Using PEB address from DbgEval: 0x%p\n", (void*)address);
          }
          else
          {
            // Try to get PEB address from TEB
            _plugin_logprintf("[Scenario Scripts] DbgEval failed, trying TEB method...\n");

            // First get TEB address - try different methods
            duint tebAddress = 0;

            // Try direct TEB evaluation
            bool tebSuccess = false;
            tebAddress = DbgEval("teb", &tebSuccess);

            // If that fails, try to get TEB via segment registers
            if (!tebSuccess || tebAddress == 0)
            {
              bool fsSuccess = false;
              duint fsBase = DbgEval("@fs", &fsSuccess);
              if (fsSuccess && fsBase != 0)
              {
                tebAddress = fsBase;
              }
            }

            // Now try to get PEB from TEB
            if (tebAddress != 0)
            {
              duint pebPtr = 0;
              // On 64-bit, the PEB pointer is at offset 0x60 from the TEB
              if (DbgMemRead(tebAddress + 0x60, &pebPtr, sizeof(pebPtr)) && pebPtr != 0)
              {
                address = pebPtr;
                useSelection = false;
                _plugin_logprintf("[Scenario Scripts] Found PEB address via TEB: 0x%p\n", (void*)address);
              }
            }

            // If methods failed, ask the user
            if (address == 0)
            {
              char inputAddress[32] = {0};
              if (ShowInputBox("PEB Address", "Enter PEB address (from PEB Parser Demo output):", inputAddress))
              {
                duint userPebAddress = 0;
                if (sscanf(inputAddress, "0x%llx", (unsigned long long*)&userPebAddress) == 1 ||
                    sscanf(inputAddress, "%llu", (unsigned long long*)&userPebAddress) == 1)
                {
                  if (userPebAddress != 0)
                  {
                    address = userPebAddress;
                    useSelection = false;
                    _plugin_logprintf("[Scenario Scripts] Using manually entered PEB address: 0x%p\n", (void*)address);
                  }
                }
              }
            }
          }
        }
        else if (structNameStr == "TEB")
        {
          // Get TEB address
          bool success = false;
          duint tebAddress = DbgEval("teb", &success);
          if (success && tebAddress != 0)
          {
            address = tebAddress;
            useSelection = false;
            _plugin_logprintf("[Scenario Scripts] Using TEB address: 0x%p\n", (void*)address);
          }
        }

        // If not a special structure, use the selected address
        if (useSelection)
        {
          duint start, end;
          if (GetSelection(&start, &end) && start != end)
            address = start;
        }

        // If we have an address, view the structure
        if (g_scenarioManager->ViewStructure(structName, address))
        {
          _plugin_logprintf("[Scenario Scripts] Structure displayed: %s at 0x%p\n", structName, address);
        }
        else
        {
          _plugin_logprintf("[Scenario Scripts] Failed to display structure: %s\n", structName);
        }
      }
    }
    break;
  }

  case MENU_AUTO_IMPORT:
    g_autoImportEnabled = !g_autoImportEnabled;
    _plugin_menuentrysetchecked(g_hMenu, MENU_AUTO_IMPORT, g_autoImportEnabled);
    if (g_scenarioManager)
    {
      g_scenarioManager->SetAutoImportEnabled(g_autoImportEnabled);
      // Settings are automatically saved in SetAutoImportEnabled
    }
    _plugin_logprintf("[Scenario Scripts] Auto import analysis %s\n", g_autoImportEnabled ? "enabled" : "disabled");

    // If enabled, run import analysis immediately
    if (g_autoImportEnabled && g_scenarioManager)
    {
      _plugin_logprintf("[Scenario Scripts] Running import analysis...\n");

      // Try kernel32.dll first as it's most likely to have useful structures
      if (g_scenarioManager->AnalyzeImports("kernel32.dll"))
      {
        _plugin_logprintf("[Scenario Scripts] Import analysis completed successfully for kernel32.dll\n");
      }
      else
      {
        _plugin_logprintf("[Scenario Scripts] Could not analyze kernel32.dll, trying ntdll.dll...\n");

        // Try ntdll.dll as an alternative
        if (g_scenarioManager->AnalyzeImports("ntdll.dll"))
        {
          _plugin_logprintf("[Scenario Scripts] Import analysis completed successfully for ntdll.dll\n");
        }
        else
        {
          // As a last resort, try the main module
          _plugin_logprintf("[Scenario Scripts] Could not analyze system DLLs, trying main module...\n");
          if (g_scenarioManager->AnalyzeImports(""))  // Empty string uses main module
          {
            _plugin_logprintf("[Scenario Scripts] Import analysis completed successfully for main module\n");
          }
          else
          {
            _plugin_logprintf("[Scenario Scripts] Import analysis failed. No suitable modules found.\n");
            _plugin_logprintf("[Scenario Scripts] Try to load the target application in the debugger first.\n");
          }
        }
      }
    }
    break;

  case MENU_CLEAR:
    if (g_scenarioManager)
    {
      g_scenarioManager->ClearScripts();
      g_scenarioManager->ClearStructures();
      _plugin_logprintf("[Scenario Scripts] All data cleared\n");
    }
    break;

  case MENU_HELP:
  {
    _plugin_logprintf("\n===== Scenario Scripts Plugin Help =====\n");
    _plugin_logprintf("This plugin allows you to load and execute script scenarios for x64dbg.\n");
    _plugin_logprintf("It also provides structure viewing capabilities for easier debugging.\n\n");
    _plugin_logprintf("Menu items:\n");
    _plugin_logprintf("- Load Script/Structure Database: Load scripts or structure definitions from files\n");
    _plugin_logprintf("- View Structure: Display and navigate a structure at a specified memory address\n");
    _plugin_logprintf("- Auto Import Analysis: Automatically analyze imports to find related structures\n");
    _plugin_logprintf("- Clear Data: Remove all loaded scripts and structures\n");
    _plugin_logprintf("==========================================\n");
    break;
  }

  case MENU_LIST_STRUCTURES:
  {
    if (g_scenarioManager)
    {
      _plugin_logprintf("\n===== Available Structures =====\n");

      // Display structures from our database
      const auto& structures = g_scenarioManager->GetLoadedStructures();
      if (!structures.empty())
      {
        _plugin_logprintf("\nLoaded structures from database:\n");
        int count = 0;
        for (const auto& pair : structures)
        {
          _plugin_logprintf("  %s\n", pair.first.c_str());
          count++;
        }
        _plugin_logprintf("Total database structures: %d\n", count);
      }
      else
      {
        _plugin_logprintf("No structures loaded in database. Use 'Load Script/Structure Database' to load some.\n");

        // If no structures are loaded, offer to load basic Windows structures
        _plugin_logprintf("\n[Scenario Scripts] Would you like to load basic Windows structures?\n");
        if (MessageBoxA(GuiGetWindowHandle(), "No structures are currently loaded. Would you like to load basic Windows structures?",
                      "Load Basic Windows Structures", MB_YESNO | MB_ICONQUESTION) == IDYES)
        {
          if (g_scenarioManager->LoadBasicWindowsStructures())
          {
            _plugin_logprintf("[Scenario Scripts] Basic Windows structures loaded successfully!\n");

            // Show the structures that were just loaded
            const auto& newStructures = g_scenarioManager->GetLoadedStructures();
            _plugin_logprintf("\nLoaded Windows structures:\n");
            int count = 0;
            for (const auto& pair : newStructures)
            {
              _plugin_logprintf("  %s\n", pair.first.c_str());
              count++;
            }
            _plugin_logprintf("Total Windows structures: %d\n", count);
          }
          else
          {
            _plugin_logprintf("[Scenario Scripts] Failed to load basic Windows structures\n");
          }
        }
      }

      // Try to enumerate structures from debugger
      _plugin_logprintf("\nStructures available from debugger:\n");
      g_scenarioManager->ViewStructure("", 0); // Empty name lists all structures

      _plugin_logprintf("================================\n");
    }
    break;
  }
  }
}

// DLL entry point
BOOL APIENTRY DllMain(HMODULE hModule, DWORD reason, LPVOID reserved)
{
  switch (reason)
  {
  case DLL_PROCESS_ATTACH:
    break;
  case DLL_PROCESS_DETACH:
    break;
  case DLL_THREAD_ATTACH:
    break;
  case DLL_THREAD_DETACH:
    break;
  }
  return TRUE;
}
