#include "scenario_scripts.h"
#include <fstream>
#include <sstream>
#include <algorithm>
#include <regex>
#include <filesystem>

// Add Windows.h first for basic Windows types
#include <Windows.h>
// Then include x64dbg plugin SDK
#include "../pluginsdk/_plugins.h"
#include "../pluginsdk/bridgemain.h"
#include "../pluginsdk/_scriptapi_module.h"
#include "../pluginsdk/_scriptapi_debug.h"

// Global instance
ScenarioManager *g_scenarioManager = nullptr;
// Reference to the plugin handle defined in plugin.cpp
extern int g_hPlugin;

// Add callback function for EnumStructs after the global variables
// Callback function for enumerating structures
static std::vector<std::string> g_availableStructures;
static void CBSTRUCTENUMPROC(const char *structName, void *userdata)
{
    if (structName && *structName)
    {
        g_availableStructures.push_back(structName);
        // Also output to log if requested
        if (userdata && *(bool *)userdata == true)
        {
            _plugin_logprintf("  %s\n", structName);
        }
    }
}

// Helper functions for script execution
bool Script_Execute(const char *script)
{
    return DbgCmdExec(script);
}

bool Script_ExecuteCommand(const char *command, char *result)
{
    return DbgCmdExec(command);
}

// For the settings functions, use BridgeSettings functions:
bool SaveSettings(const char *section, const char *key, const char *value)
{
    return BridgeSettingSet(section, key, value);
}

bool SaveSettingsUint(const char *section, const char *key, duint value)
{
    return BridgeSettingSetUint(section, key, value);
}

bool GetSettings(const char *section, const char *key, char *value)
{
    return BridgeSettingGet(section, key, value);
}

bool GetSettingsUint(const char *section, const char *key, duint *value)
{
    return BridgeSettingGetUint(section, key, value);
}

bool FlushSettings()
{
    return BridgeSettingFlush();
}

// Constructor
ScenarioManager::ScenarioManager() : m_autoImportEnabled(false)
{
}

// Destructor
ScenarioManager::~ScenarioManager()
{
    Shutdown();
}

// Initialize the manager
bool ScenarioManager::Initialize()
{
    // Load saved settings
    LoadSettings();

    // If we have a saved structure database path, try to load it
    if (!m_lastStructureDbPath.empty())
    {
        LoadStructureDatabase(m_lastStructureDbPath);
    }

    // If no structures were loaded and there's no saved path, load basic Windows structures
    if (m_structures.empty())
    {
        _plugin_logprintf("[Scenario Scripts] No structures loaded. Loading basic Windows structures...\n");
        LoadBasicWindowsStructures();
    }

    return true;
}

// Shutdown the manager
void ScenarioManager::Shutdown()
{
    // Save settings before shutdown
    SaveSettings();

    // Clear all data
    ClearScripts();
    ClearStructures();
}

// Load a script from file
bool ScenarioManager::LoadScriptFromFile(const std::string &filePath)
{
    std::ifstream file(filePath);
    if (!file.is_open())
    {
        _plugin_logprintf("[Scenario Scripts] Could not open file: %s\n", filePath.c_str());
        return false;
    }

    std::stringstream buffer;
    buffer << file.rdbuf();
    file.close();

    ScenarioScript script;
    script.path = filePath;

    // Extract filename for script name
    size_t lastSlash = filePath.find_last_of("/\\");
    if (lastSlash != std::string::npos)
    {
        script.name = filePath.substr(lastSlash + 1);
    }
    else
    {
        script.name = filePath;
    }

    script.content = buffer.str();
    script.isActive = false;

    m_loadedScripts.push_back(script);

    _plugin_logprintf("[Scenario Scripts] Loaded script: %s\n", script.name.c_str());
    return true;
}

// Execute a loaded script
bool ScenarioManager::ExecuteScript(const std::string &scriptName)
{
    for (auto &script : m_loadedScripts)
    {
        if (script.name == scriptName)
        {
            // Use x64dbg's script API to execute the script
            if (Script_Execute(script.content.c_str()))
            {
                script.isActive = true;
                _plugin_logprintf("[Scenario Scripts] Executed script: %s\n", scriptName.c_str());
                return true;
            }
            else
            {
                _plugin_logprintf("[Scenario Scripts] Failed to execute script: %s\n", scriptName.c_str());
                return false;
            }
        }
    }

    _plugin_logprintf("[Scenario Scripts] Script not found: %s\n", scriptName.c_str());
    return false;
}

// Clear all loaded scripts
void ScenarioManager::ClearScripts()
{
    m_loadedScripts.clear();
    _plugin_logprintf("[Scenario Scripts] Cleared all scripts\n");
}

// Get all loaded scripts
const std::vector<ScenarioScript> &ScenarioManager::GetLoadedScripts() const
{
    return m_loadedScripts;
}

// Load structure definitions from a database file
bool ScenarioManager::LoadStructureDatabase(const std::string &filePath)
{
    std::ifstream file(filePath);
    if (!file.is_open())
    {
        _plugin_logprintf("[Scenario Scripts] Could not open structure database: %s\n", filePath.c_str());
        return false;
    }

    m_lastStructureDbPath = filePath;

    std::string line;
    std::string currentStructure;
    bool inStructure = false;
    std::stringstream structDef;
    int braceCount = 0;

    while (std::getline(file, line))
    {
        // Skip empty lines and comments
        if (line.empty() || line[0] == '#' || line[0] == '/')
        {
            continue;
        }

        // Detect start of structure
        if (!inStructure && line.find("{") != std::string::npos)
        {
            // Extract structure name
            size_t bracePos = line.find("{");
            std::string name = line.substr(0, bracePos);
            name = std::regex_replace(name, std::regex("^\\s+|\\s+$"), ""); // Trim whitespace

            currentStructure = name;
            inStructure = true;
            braceCount = 1;
            structDef.str("");
            structDef << line << std::endl;
        }
        // In structure definition
        else if (inStructure)
        {
            structDef << line << std::endl;

            // Count braces to detect end of structure
            for (char c : line)
            {
                if (c == '{')
                    braceCount++;
                if (c == '}')
                    braceCount--;
            }

            // End of structure definition
            if (braceCount <= 0)
            {
                inStructure = false;
                std::string definition = structDef.str();

                // Parse the structure completely using our existing method
                ParseStructureDefinition(definition);

                // Get the structure from the map and ensure dependent structures are set
                auto it = m_structures.find(currentStructure);
                if (it != m_structures.end())
                {
                    it->second.dependentStructures = ExtractDependentStructures(definition);
                }

                _plugin_logprintf("[Scenario Scripts] Loaded structure: %s\n", currentStructure.c_str());
            }
        }
    }

    file.close();

    // Save settings to remember the last loaded database
    SaveSettings();

    return !m_structures.empty();
}

// View a structure at a memory address
bool ScenarioManager::ViewStructure(const std::string &structureName, duint address)
{
    // Get DbgFunctions
    const DBGFUNCTIONS *dbgFunctions = DbgFunctions();

    // If the structure name is empty or "list", enumerate all available structures
    if (structureName.empty() || structureName == "list")
    {
        _plugin_logprintf("[Scenario Scripts] Available structures:\n");

        // Clear the global structure list
        g_availableStructures.clear();

        // Flag to indicate logging is desired
        bool logStructures = true;

        // Enumerate structures if the function is available
        if (dbgFunctions && dbgFunctions->EnumStructs)
        {
            dbgFunctions->EnumStructs(CBSTRUCTENUMPROC, &logStructures);

            if (g_availableStructures.empty())
            {
                _plugin_logprintf("[Scenario Scripts] No structures available in debugger\n");
            }
            else
            {
                _plugin_logprintf("[Scenario Scripts] Total available structures in debugger: %d\n",
                                  (int)g_availableStructures.size());
            }

            // Also list structures from our database
            if (!m_structures.empty())
            {
                _plugin_logprintf("\n[Scenario Scripts] Structures from database:\n");
                for (const auto &pair : m_structures)
                {
                    _plugin_logprintf("  %s\n", pair.first.c_str());
                }
                _plugin_logprintf("[Scenario Scripts] Total structures in database: %d\n",
                                  (int)m_structures.size());
            }
            return true;
        }
        else
        {
            _plugin_logprintf("[Scenario Scripts] EnumStructs function not available\n");
            return false;
        }
    }

    // Check if the requested structure exists in the debugger
    g_availableStructures.clear();
    bool structFound = false;

    if (dbgFunctions && dbgFunctions->EnumStructs)
    {
        // First, try to enumerate structures with logging
        bool enableLogging = structureName.empty() || structureName == "list";
        if (enableLogging)
        {
            _plugin_logprintf("[Scenario Scripts] Enumerating structures in debugger...\n");
        }

        dbgFunctions->EnumStructs(CBSTRUCTENUMPROC, enableLogging ? (void*)&enableLogging : nullptr);

        // If no structures are available, suggest loading Windows structures
        if (g_availableStructures.empty() && enableLogging)
        {
            _plugin_logprintf("[Scenario Scripts] No structures found in debugger. You can:\n");
            _plugin_logprintf("[Scenario Scripts] 1. Load a structure database file using 'Load Script/Structure Database'\n");
            _plugin_logprintf("[Scenario Scripts] 2. Use 'Auto Import Analysis' to detect relevant structures\n");
            _plugin_logprintf("[Scenario Scripts] 3. Type structure definitions in the x64dbg script window\n");
            _plugin_logprintf("[Scenario Scripts] For example: struct POINT { int x; int y; };\n");
        }

        // Check if requested structure is in the enumerated list
        structFound = !structureName.empty() &&
                     std::find(g_availableStructures.begin(), g_availableStructures.end(), structureName) != g_availableStructures.end();
    }

    // Find in our own database
    auto it = m_structures.find(structureName);
    bool inOurDatabase = (it != m_structures.end());

    if (inOurDatabase)
    {
        _plugin_logprintf("[Scenario Scripts] Structure '%s' found in database\n", structureName.c_str());
    }

    // Not found anywhere
    if (!structFound && !inOurDatabase)
    {
        _plugin_logprintf("[Scenario Scripts] Structure not found: %s\n", structureName.c_str());
        _plugin_logprintf("[Scenario Scripts] Use empty name or 'list' to see all available structures\n");
        return false;
    }

    // Choose which source to use (prefer debugger's definition if available)
    bool useDebuggerStruct = structFound;

    // Special handling for known Windows structures
    if (address == 0)
    {
        // Handle special structures with known locations
        if (structureName == "PEB")
        {
            address = ResolvePebAddress();
            _plugin_logprintf("[Scenario Scripts] Using PEB address: 0x%p\n", (void *)address);
        }
        else if (structureName == "TEB")
        {
            address = ResolveTebAddress();
            _plugin_logprintf("[Scenario Scripts] Using TEB address: 0x%p\n", (void *)address);
        }
        else if (structureName.find("PROCESS_") != std::string::npos ||
                 structureName.find("SYSTEM_") != std::string::npos ||
                 structureName.find("MEMORY_") != std::string::npos)
        {
            // Try to find valid data section address from main module
            address = FindValidDataSectionAddress();
            _plugin_logprintf("[Scenario Scripts] Using data section address: 0x%p\n", (void *)address);
        }

        // If still no address, try to get current selection from dump window
        if (address == 0)
        {
            // Try to get selected address from dump window instead of code window
            address = DbgValFromString("sdump");

            // If no dump selection, use current data window address
            if (address == 0)
                address = DbgValFromString("cdata");

            // Last resort - don't use instruction pointer as it's usually code, not data
            if (address == 0)
            {
                // Allocate memory in the debuggee for demonstration purposes
                std::stringstream allocCmd;
                allocCmd << "alloc 0x1000";
                char result[MAX_PATH] = {0};
                if (Script_ExecuteCommand(allocCmd.str().c_str(), result))
                {
                    // Parse the allocation result to get the address
                    std::string resultStr(result);
                    size_t posAddr = resultStr.find("0x");
                    if (posAddr != std::string::npos)
                    {
                        std::string addrStr = resultStr.substr(posAddr);
                        address = DbgValFromString(addrStr.c_str());
                        _plugin_logprintf("[Scenario Scripts] Allocated memory for structure display: 0x%p\n", (void *)address);
                    }
                }

                // If allocation failed, warn the user
                if (address == 0)
                {
                    _plugin_logprintf("[Scenario Scripts] WARNING: Could not determine a valid data address. Please specify an address.\n");
                    return false;
                }
            }
        }
    }

    if (address == 0)
    {
        _plugin_logprintf("[Scenario Scripts] Invalid address for structure view. Please specify a valid memory address.\n");
        return false;
    }

    // Check if address is in a valid memory region
    duint size = 0;
    if (!DbgMemIsValidReadPtr(address))
    {
        _plugin_logprintf("[Scenario Scripts] WARNING: Address 0x%p is not a valid readable memory location.\n", (void *)address);
        _plugin_logprintf("[Scenario Scripts] The structure will be displayed, but values may be incorrect.\n");
    }

    // Load the structure into x64dbg's structure view
    std::stringstream structCmd;
    structCmd << "Structures.Load " << structureName << ", 0x" << std::hex << address;
    std::string command = structCmd.str();
    bool result = Script_Execute(command.c_str());

    if (result)
    {
        _plugin_logprintf("[Scenario Scripts] Loaded structure %s at address 0x%p\n", structureName.c_str(), (void *)address);

        // For TypeSystem display, try to use VisitType command if available
        std::stringstream visitCmd;
        visitCmd << "VisitType \"" << structureName << "\", 0x" << std::hex << address;
        std::string visitTypeCmd = visitCmd.str();
        if (Script_Execute(visitTypeCmd.c_str()))
        {
            _plugin_logprintf("[Scenario Scripts] Also loaded structure in Type view\n");
        }

        // Navigate to the structure address in the memory dump
        std::stringstream ss;
        ss << "dump 0x" << std::hex << address;
        std::string dumpCommand = ss.str();
        if (Script_Execute(dumpCommand.c_str()))
        {
            _plugin_logprintf("[Scenario Scripts] Navigated to structure memory in dump view\n");

            // If we have the structure in our database, try to highlight its bytes
            if (inOurDatabase)
            {
                size_t structSize = CalculateStructureSize(it->second);
                if (structSize > 0)
                {
                    // Highlight the bytes representing the structure
                    std::stringstream hexCmd;
                    hexCmd << "SetHexSelection 0x" << std::hex << address << ", 0x" << std::hex << (address + structSize - 1);
                    std::string highlightCmd = hexCmd.str();
                    if (Script_Execute(highlightCmd.c_str()))
                    {
                        _plugin_logprintf("[Scenario Scripts] Highlighted structure bytes in dump (%llu bytes)\n", (unsigned long long)structSize);
                    }
                }
            }
            else if (useDebuggerStruct)
            {
                // For debugger-provided structures, use a reasonable size for highlighting
                // You could enhance this by querying the structure size via x64dbg API if available
                size_t defaultSize = 64; // Default to 64 bytes

                std::stringstream hexCmd;
                hexCmd << "SetHexSelection 0x" << std::hex << address << ", 0x" << std::hex << (address + defaultSize - 1);
                std::string highlightCmd = hexCmd.str();
                if (Script_Execute(highlightCmd.c_str()))
                {
                    _plugin_logprintf("[Scenario Scripts] Highlighted structure bytes in dump (estimated size)\n");
                }
            }
        }

        // Add code to display the structure contents
        _plugin_logprintf("\n=== Structure %s Contents ===\n", structureName.c_str());

        // If using debugger structure, use simpler display
        if (useDebuggerStruct)
        {
            _plugin_logprintf("[Scenario Scripts] Structure address: 0x%p\n", (void *)address);
            _plugin_logprintf("[Scenario Scripts] Using debugger's structure definition\n");
            _plugin_logprintf("[Scenario Scripts] View the structure in x64dbg's Structures tab for details\n");
            return true;
        }

        // Get the structure definition from our database
        const StructureDefinition &structDef = it->second;

        // Display the structure members
        DisplayStructureMembers(structDef, address);

        _plugin_logprintf("=========================\n");
        _plugin_logprintf("[Scenario Scripts] Check the Structures tab in x64dbg to view this structure in the GUI\n");

        return true;
    }
    else
    {
        _plugin_logprintf("[Scenario Scripts] Failed to load structure %s\n", structureName.c_str());
        return false;
    }
}

// Analyze imports to find relevant structures
bool ScenarioManager::AnalyzeImports(const std::string &moduleName)
{
    // Используем имя модуля как есть, без автоматического добавления .dll
    std::string module = moduleName.empty() ? "kernel32.dll" : moduleName;

    _plugin_logprintf("[Scenario Scripts] Analyzing imports for module: %s\n", module.c_str());

    // Find imported functions using SDK API
    std::vector<std::string> importedFunctions = FindImportedFunctions(module);
    if (importedFunctions.empty())
    {
        _plugin_logprintf("[Scenario Scripts] No imports found for module: %s\n", module.c_str());

        // Если не удалось найти импорты, попробуем показать доступные модули
        _plugin_logprintf("[Scenario Scripts] Listing loaded modules...\n");

        ListInfo moduleList;
        if (Script::Module::GetList(&moduleList))
        {
            _plugin_logprintf("[Scenario Scripts] Try one of these loaded modules:\n");
            for (int i = 0; i < std::min<int>(5, moduleList.count); i++)
            {
                const Script::Module::ModuleInfo* modules =
                    reinterpret_cast<const Script::Module::ModuleInfo*>(moduleList.data);
                _plugin_logprintf("  %s\n", modules[i].name);
            }

            if (moduleList.count > 5)
            {
                _plugin_logprintf("  ... and %d more modules\n", moduleList.count - 5);
            }

            BridgeFree(moduleList.data);
        }

        return false;
    }

    // Map functions to structures
    std::vector<std::string> relevantStructures = MapFunctionsToStructures(importedFunctions);

    if (relevantStructures.empty())
    {
        _plugin_logprintf("[Scenario Scripts] No relevant structures found for imports from: %s\n", module.c_str());
        _plugin_logprintf("[Scenario Scripts] Total analyzed imports: %d\n", (int)importedFunctions.size());

        // Покажем некоторые импорты для отладки
        _plugin_logprintf("[Scenario Scripts] Sample imports:\n");
        int count = 0;
        for (const auto& func : importedFunctions)
        {
            _plugin_logprintf("  %s\n", func.c_str());
            if (++count >= 10) {
                _plugin_logprintf("  ... and %d more\n", (int)importedFunctions.size() - 10);
                break;
            }
        }

        return false;
    }

    // Добавим найденные структуры в нашу базу, если их там еще нет
    int addedCount = 0;
    for (const auto &structName : relevantStructures)
    {
        // Проверяем, существует ли структура в базе
        if (m_structures.find(structName) == m_structures.end())
        {
            // Если нет, создаем базовый шаблон структуры
            std::string structTemplate = structName + " { /* Structure members will be filled when defined */ }";
            if (ParseStructureDefinition(structTemplate))
            {
                addedCount++;
                _plugin_logprintf("[Scenario Scripts] Added structure template: %s\n", structName.c_str());
            }
        }
    }

    // Display the results
    _plugin_logprintf("[Scenario Scripts] Found %d relevant structures for imports from %s:\n",
                      (int)relevantStructures.size(), module.c_str());

    for (const auto &structName : relevantStructures)
    {
        _plugin_logprintf("  - %s\n", structName.c_str());
    }

    if (addedCount > 0)
    {
        _plugin_logprintf("[Scenario Scripts] Added %d new structure templates to the database\n", addedCount);
        _plugin_logprintf("[Scenario Scripts] Use 'List All Structures' to see them\n");
    }

    return true;
}

// Get structures associated with a module
std::vector<std::string> ScenarioManager::GetStructuresForModule(const std::string &moduleName)
{
    // Find imported functions for the module
    std::vector<std::string> importedFunctions = FindImportedFunctions(moduleName);

    // Map to structures
    return MapFunctionsToStructures(importedFunctions);
}

// Clear all loaded structures
void ScenarioManager::ClearStructures()
{
    m_structures.clear();
    m_functionToStructureMap.clear();
    _plugin_logprintf("[Scenario Scripts] Cleared all structures\n");
}

// Save settings
bool ScenarioManager::SaveSettings()
{
    // Use x64dbg's settings API to save plugin settings
    if (!m_lastStructureDbPath.empty())
    {
        // Use the global function directly to avoid confusion with the member function
        ::SaveSettings("ScenarioScripts", "LastStructureDbPath", m_lastStructureDbPath.c_str());
    }

    ::SaveSettingsUint("ScenarioScripts", "AutoImportEnabled", m_autoImportEnabled ? 1 : 0);
    ::FlushSettings();

    return true;
}

// Load settings
bool ScenarioManager::LoadSettings()
{
    char buffer[MAX_PATH] = "";
    if (GetSettings("ScenarioScripts", "LastStructureDbPath", buffer))
    {
        m_lastStructureDbPath = buffer;
    }

    duint enabled = 0;
    if (GetSettingsUint("ScenarioScripts", "AutoImportEnabled", &enabled))
    {
        m_autoImportEnabled = (enabled != 0);
    }

    return true;
}

// Set auto import enabled state
void ScenarioManager::SetAutoImportEnabled(bool enabled)
{
    m_autoImportEnabled = enabled;
    SaveSettings();
}

// Get auto import enabled state
bool ScenarioManager::GetAutoImportEnabled() const
{
    return m_autoImportEnabled;
}

// Get all loaded structures
const std::map<std::string, StructureDefinition>& ScenarioManager::GetLoadedStructures() const
{
    return m_structures;
}

// Parse a structure definition
bool ScenarioManager::ParseStructureDefinition(const std::string &definition)
{
    // Extract structure name
    std::regex nameRegex("^\\s*([A-Za-z0-9_]+)\\s*\\{");
    std::smatch nameMatch;
    if (!std::regex_search(definition, nameMatch, nameRegex))
    {
        return false;
    }

    std::string structName = nameMatch[1];

    // Create structure
    StructureDefinition structDef(structName, definition);

    // Extract members - improved regex to better match various field formats
    std::regex memberRegex("\\s+([A-Za-z0-9_]+)\\s+([A-Za-z0-9_\\[\\]]+)\\s*;");
    std::string::const_iterator searchStart(definition.cbegin());
    std::smatch memberMatch;

    while (std::regex_search(searchStart, definition.cend(), memberMatch, memberRegex))
    {
        std::string memberType = memberMatch[1];
        std::string memberName = memberMatch[2];

        // Remove any array size information from the member name
        size_t bracketPos = memberName.find('[');
        if (bracketPos != std::string::npos)
        {
            memberName = memberName.substr(0, bracketPos);
        }

        structDef.members.push_back(memberName);

        // Store type information too for better memory reading
        structDef.memberTypes.push_back(memberType);

        // Check if member type is another structure
        if (m_structures.find(memberType) != m_structures.end())
        {
            structDef.dependentStructures.push_back(memberType);
        }

        searchStart = memberMatch.suffix().first;
    }

    // Add to map
    m_structures[structName] = structDef;

    return true;
}

// Extract dependent structures from a definition
std::vector<std::string> ScenarioManager::ExtractDependentStructures(const std::string &definition)
{
    std::vector<std::string> dependencies;

    // This is a simplified implementation
    // A real-world version would need more sophisticated parsing

    // Look for known structures in the definition
    for (const auto &pair : m_structures)
    {
        const std::string &structName = pair.first;

        // Simple check for structure name followed by variable name
        std::regex structRegex("\\b" + structName + "\\s+([A-Za-z0-9_]+)\\s*;");

        if (std::regex_search(definition, structRegex))
        {
            dependencies.push_back(structName);
        }
    }

    return dependencies;
}

// Find imported functions from a module
std::vector<std::string> ScenarioManager::FindImportedFunctions(const std::string &moduleName)
{
    std::vector<std::string> functions;

    // Получаем информацию о модуле
    Script::Module::ModuleInfo moduleInfo;
    bool moduleFound = false;

    if (!moduleName.empty())
    {
        // Преобразуем std::string в char*
        char modName[MAX_MODULE_SIZE] = {0};
        strncpy_s(modName, moduleName.c_str(), _TRUNCATE);

        // Получаем информацию о модуле по имени
        moduleFound = Script::Module::InfoFromName(modName, &moduleInfo);

        if (!moduleFound)
        {
            _plugin_logprintf("[Scenario Scripts] Module not found: %s\n", moduleName.c_str());
            _plugin_logprintf("[Scenario Scripts] Ensure module is loaded in the debuggee\n");

            // Попробуем показать список всех доступных модулей
            ListInfo moduleList;
            if (Script::Module::GetList(&moduleList))
            {
                _plugin_logprintf("[Scenario Scripts] Available modules:\n");

                int count = 0;
                for (int i = 0; i < moduleList.count; i++)
                {
                    const Script::Module::ModuleInfo* modules =
                        reinterpret_cast<const Script::Module::ModuleInfo*>(moduleList.data);
                    _plugin_logprintf("  %s (Base: 0x%p)\n", modules[i].name, (void*)modules[i].base);
                    count++;

                    // Ограничим вывод 10 модулями
                    if (count >= 10)
                    {
                        _plugin_logprintf("  ... and %d more\n", moduleList.count - 10);
                        break;
                    }
                }

                BridgeFree(moduleList.data);
            }

            return functions;
        }
    }
    else
    {
        // Если имя не указано, используем основной модуль
        moduleFound = Script::Module::GetMainModuleInfo(&moduleInfo);

        if (!moduleFound)
        {
            _plugin_logprintf("[Scenario Scripts] Could not get main module info\n");
            return functions;
        }
    }

    // Теперь получаем импорты
    _plugin_logprintf("[Scenario Scripts] Getting imports for module: %s (Base: 0x%p)\n",
                     moduleInfo.name, (void*)moduleInfo.base);

    ListInfo importList;
    if (Script::Module::GetImports(&moduleInfo, &importList))
    {
        _plugin_logprintf("[Scenario Scripts] Found %d imports\n", importList.count);

        for (int i = 0; i < importList.count; i++)
        {
            const Script::Module::ModuleImport* imports =
                reinterpret_cast<const Script::Module::ModuleImport*>(importList.data);

            if (imports[i].name[0] != '\0')
            {
                functions.push_back(imports[i].name);

                // Log some of the first imports (maximum 10)
                if (i < 10)
                {
                    _plugin_logprintf("[Scenario Scripts] Import #%d: %s (IAT VA: 0x%p)\n",
                                     i+1, imports[i].name, (void*)imports[i].iatVa);
                }
                else if (i == 10)
                {
                    _plugin_logprintf("[Scenario Scripts] ... and %d more imports\n",
                                     importList.count - 10);
                }
            }
        }

        BridgeFree(importList.data);
    }
    else
    {
        _plugin_logprintf("[Scenario Scripts] Failed to get imports for module: %s\n",
                         moduleInfo.name);
    }

    return functions;
}

// Map functions to structures
std::vector<std::string> ScenarioManager::MapFunctionsToStructures(const std::vector<std::string> &functions)
{
    std::vector<std::string> structures;

    // This is where we'd implement the logic to map functions to structures
    // For now, this is a simplified implementation with some common mappings

    // Basic mapping for common Windows API functions
    std::map<std::string, std::vector<std::string>> functionToStructs = {
        {"CreateProcess", {"PROCESS_INFORMATION", "STARTUPINFO"}},
        {"ReadProcessMemory", {"MEMORY_BASIC_INFORMATION"}},
        {"VirtualQuery", {"MEMORY_BASIC_INFORMATION"}},
        {"GetThreadContext", {"CONTEXT"}},
        {"SetThreadContext", {"CONTEXT"}},
        {"CreateFile", {"SECURITY_ATTRIBUTES"}},
        {"GetFileInformation", {"BY_HANDLE_FILE_INFORMATION"}},
        {"FindFirstFile", {"WIN32_FIND_DATA"}},
        {"FindNextFile", {"WIN32_FIND_DATA"}},
        {"GetSystemInfo", {"SYSTEM_INFO"}},
        {"CreateWindowEx", {"WNDCLASSEX"}},
        {"RegisterClassEx", {"WNDCLASSEX"}},
        {"GetMessage", {"MSG"}},
        {"PeekMessage", {"MSG"}},
        {"ReadFile", {"OVERLAPPED"}},
        {"WriteFile", {"OVERLAPPED"}},
        {"CoCreateInstance", {"IID", "CLSID"}}};

    // For each imported function, check if it maps to any structures
    for (const auto &func : functions)
    {
        auto it = functionToStructs.find(func);
        if (it != functionToStructs.end())
        {
            // Add associated structures to the result
            for (const auto &structName : it->second)
            {
                // Only add if we haven't already added it
                if (std::find(structures.begin(), structures.end(), structName) == structures.end())
                {
                    structures.push_back(structName);
                }
            }
        }
    }

    return structures;
}

// Find a valid address in the data section of the main module
duint ScenarioManager::FindValidDataSectionAddress()
{
    // Get the base address of the main module
    duint moduleBase = Script::Module::GetMainModuleBase();
    if (!moduleBase)
        return 0;

    // Get module size
    duint moduleSize = Script::Module::GetMainModuleSize();
    if (moduleSize == 0)
        return 0;

    _plugin_logprintf("[Scenario Scripts] Main module base: 0x%p, size: 0x%llx\n",
                      (void*)moduleBase, (unsigned long long)moduleSize);

    // Try a few offsets from the module base that are likely to be in data sections
    // Common offsets for data sections in Windows PE files
    const duint offsets[] = {
        0x1000,  // Often the start of .text section
        0x2000,  // Often after .text section
        0x3000,  // May be in .data section
        0x4000,  // May be in .data section
        0x5000,  // Another possible data section
        0x10000  // Far enough to be past code sections
    };

    // Try each offset
    for (size_t i = 0; i < sizeof(offsets) / sizeof(offsets[0]); i++)
    {
        duint addr = moduleBase + offsets[i];

        // Make sure we're still within the module
        if (addr < moduleBase + moduleSize)
        {
            // Check if it's readable
            if (DbgMemIsValidReadPtr(addr))
            {
                _plugin_logprintf("[Scenario Scripts] Found valid memory at offset 0x%llx\n",
                                 (unsigned long long)offsets[i]);
                return addr;
            }
        }
    }

    // If no good offset found, try allocation
    _plugin_logprintf("[Scenario Scripts] No valid data section found, trying allocation...\n");

    // Try to allocate memory in the debuggee
    std::stringstream allocCmd;
    allocCmd << "alloc 0x1000";
    char result[MAX_PATH] = {0};
    if (Script_ExecuteCommand(allocCmd.str().c_str(), result))
    {
        // Parse the allocation result to get the address
        std::string resultStr(result);
        size_t posAddr = resultStr.find("0x");
        if (posAddr != std::string::npos)
        {
            std::string addrStr = resultStr.substr(posAddr);
            duint allocAddr = DbgValFromString(addrStr.c_str());
            if (allocAddr && DbgMemIsValidReadPtr(allocAddr))
            {
                _plugin_logprintf("[Scenario Scripts] Allocated memory for structure view: 0x%p\n", (void*)allocAddr);
                return allocAddr;
            }
        }
    }

    // Last resort - try using a global DLL address
    duint ntdllBase = Script::Module::BaseFromName("ntdll.dll");
    if (ntdllBase && DbgMemIsValidReadPtr(ntdllBase + 0x1000))
    {
        _plugin_logprintf("[Scenario Scripts] Using ntdll.dll as fallback: 0x%p\n", (void*)(ntdllBase + 0x1000));
        return ntdllBase + 0x1000;
    }

    // If all else fails
    _plugin_logprintf("[Scenario Scripts] Could not find a valid data address\n");
    return 0;
}

// Load basic Windows structures
bool ScenarioManager::LoadBasicWindowsStructures()
{
    bool result = false;
    int structureCount = 0;

    // Define some basic Windows structures
    std::vector<std::string> basicStructures = {
        // Point structure
        "POINT { LONG x; LONG y; }",

        // Rectangle structure
        "RECT { LONG left; LONG top; LONG right; LONG bottom; }",

        // Basic Windows handles and information structures
        "SECURITY_ATTRIBUTES { DWORD nLength; LPVOID lpSecurityDescriptor; BOOL bInheritHandle; }",

        // File information
        "WIN32_FIND_DATA { DWORD dwFileAttributes; FILETIME ftCreationTime; FILETIME ftLastAccessTime; FILETIME ftLastWriteTime; DWORD nFileSizeHigh; DWORD nFileSizeLow; DWORD dwReserved0; DWORD dwReserved1; CHAR cFileName[260]; CHAR cAlternateFileName[14]; }",

        // Process structures
        "STARTUPINFO { DWORD cb; LPSTR lpReserved; LPSTR lpDesktop; LPSTR lpTitle; DWORD dwX; DWORD dwY; DWORD dwXSize; DWORD dwYSize; DWORD dwXCountChars; DWORD dwYCountChars; DWORD dwFillAttribute; DWORD dwFlags; WORD wShowWindow; WORD cbReserved2; LPBYTE lpReserved2; HANDLE hStdInput; HANDLE hStdOutput; HANDLE hStdError; }",

        "PROCESS_INFORMATION { HANDLE hProcess; HANDLE hThread; DWORD dwProcessId; DWORD dwThreadId; }",

        // Memory information
        "MEMORY_BASIC_INFORMATION { PVOID BaseAddress; PVOID AllocationBase; DWORD AllocationProtect; SIZE_T RegionSize; DWORD State; DWORD Protect; DWORD Type; }",

        // PE format structures
        "IMAGE_DOS_HEADER { WORD e_magic; WORD e_cblp; WORD e_cp; WORD e_crlc; WORD e_cparhdr; WORD e_minalloc; WORD e_maxalloc; WORD e_ss; WORD e_sp; WORD e_csum; WORD e_ip; WORD e_cs; WORD e_lfarlc; WORD e_ovno; WORD e_res[4]; WORD e_oemid; WORD e_oeminfo; WORD e_res2[10]; LONG e_lfanew; }",

        "IMAGE_FILE_HEADER { WORD Machine; WORD NumberOfSections; DWORD TimeDateStamp; DWORD PointerToSymbolTable; DWORD NumberOfSymbols; WORD SizeOfOptionalHeader; WORD Characteristics; }",

        // NT specific structures
        "UNICODE_STRING { USHORT Length; USHORT MaximumLength; PWSTR Buffer; }"
    };

    // Try to parse and add each structure
    for (const auto& structDef : basicStructures)
    {
        if (ParseStructureDefinition(structDef))
        {
            structureCount++;
            result = true;
        }
    }

    if (result)
    {
        _plugin_logprintf("[Scenario Scripts] Loaded %d basic Windows structures\n", structureCount);
    }

    return result;
}
