#pragma once

#include <windows.h>
#include <vector>
#include <map>
#include <string>
#include "../pluginsdk/_plugins.h"
#include "../pluginsdk/_dbgfunctions.h"
#include "../pluginsdk/_scriptapi.h"

// Structure to represent a data structure definition
struct StructureDefinition {
    std::string name;
    std::string definition;
    std::vector<std::string> members;
    std::vector<std::string> dependentStructures;
    std::vector<std::string> memberTypes;

    StructureDefinition() {}
    StructureDefinition(const std::string& n, const std::string& def)
        : name(n), definition(def) {}
};

// Structure to store information about a loaded script
struct ScenarioScript {
    std::string name;
    std::string path;
    std::string content;
    bool isActive;

    ScenarioScript() : isActive(false) {}
};

// Class to manage scenario scripts and structure definitions
class ScenarioManager {
public:
    ScenarioManager();
    ~ScenarioManager();

    // Initialization and shutdown
    bool Initialize();
    void Shutdown();

    // Script management
    bool LoadScriptFromFile(const std::string& filePath);
    bool ExecuteScript(const std::string& scriptName);
    void ClearScripts();
    const std::vector<ScenarioScript>& GetLoadedScripts() const;

    // Structure management
    bool LoadStructureDatabase(const std::string& filePath);
    bool LoadBasicWindowsStructures();
    bool ViewStructure(const std::string& structureName, duint address = 0);
    bool AnalyzeImports(const std::string& moduleName = "");
    std::vector<std::string> GetStructuresForModule(const std::string& moduleName);
    void ClearStructures();
    const std::map<std::string, StructureDefinition>& GetLoadedStructures() const;

    // Settings management
    bool SaveSettings();
    bool LoadSettings();
    void SetAutoImportEnabled(bool enabled);
    bool GetAutoImportEnabled() const;

private:
    // Helper methods for parsing structure definitions
    bool ParseStructureDefinition(const std::string& definition);
    std::vector<std::string> ExtractDependentStructures(const std::string& definition);

    // Helper methods for import analysis
    std::vector<std::string> FindImportedFunctions(const std::string& moduleName);
    std::vector<std::string> MapFunctionsToStructures(const std::vector<std::string>& functions);

    // Helper methods for structure display
    duint ResolvePebAddress();
    duint ResolveTebAddress();
    duint FindValidDataSectionAddress();
    size_t GetTypeSize(const std::string& typeName);
    void DisplayStructureMembers(const StructureDefinition& structDef, duint baseAddress);
    void DisplayMemberValue(const unsigned char* buffer, const std::string& typeName, duint address);
    size_t CalculateStructureSize(const StructureDefinition& structDef);

    // Internal data storage
    std::vector<ScenarioScript> m_loadedScripts;
    std::map<std::string, StructureDefinition> m_structures;
    std::map<std::string, std::vector<std::string>> m_functionToStructureMap;

    // Settings
    bool m_autoImportEnabled;
    std::string m_lastStructureDbPath;
};

// Global instance of the manager
extern ScenarioManager* g_scenarioManager;
