#include "scenario_scripts.h"
#include <regex>

// Get size of a type in bytes
size_t ScenarioManager::GetTypeSize(const std::string& typeName)
{
    // Basic types
    if (typeName == "BYTE" || typeName == "CHAR" || typeName == "BOOL")
        return 1;
    else if (typeName == "WORD" || typeName == "SHORT")
        return 2;
    else if (typeName == "DWORD" || typeName == "LONG" || typeName == "FLOAT")
        return 4;
    else if (typeName == "QWORD" || typeName == "LONGLONG" || typeName == "DOUBLE" ||
             typeName.find("HANDLE") != std::string::npos ||
             typeName.find("LPSTR") != std::string::npos ||
             typeName.find("LPBYTE") != std::string::npos ||
             typeName.find("FILETIME") != std::string::npos ||
             typeName.find("LP") == 0 ||
             typeName == "PVOID" || typeName == "SIZE_T" || typeName == "LARGE_INTEGER" ||
             typeName.find("ULONGLONG") != std::string::npos ||
             typeName == "UNICODE_STRING")
        return sizeof(void*);  // Pointer size or 8 bytes for 64-bit types

    // Unknown type - assume pointer size
    return sizeof(void*);
}

// Helper to resolve PEB address
duint ScenarioManager::ResolvePebAddress()
{
    _plugin_logprintf("[Scenario Scripts] Trying to locate PEB address...\n");

    // Method 1: Try using DbgEval to get the PEB directly
    bool success = false;
    duint pebAddress = DbgEval("peb", &success);
    if (success && pebAddress != 0)
    {
        _plugin_logprintf("[Scenario Scripts] Using PEB address from DbgEval: 0x%p\n", (void*)pebAddress);
        return pebAddress;
    }

    // Method 2: Try to get the PEB address from TEB
    _plugin_logprintf("[Scenario Scripts] DbgEval failed, trying TEB method...\n");

    // Get TEB address first
    duint tebAddress = ResolveTebAddress();

    // Now try to get PEB from TEB
    if (tebAddress != 0)
    {
        duint pebPtr = 0;
        // On 64-bit, the PEB pointer is at offset 0x60 from the TEB
        if (DbgMemRead(tebAddress + 0x60, &pebPtr, sizeof(pebPtr)) && pebPtr != 0)
        {
            _plugin_logprintf("[Scenario Scripts] Found PEB address via TEB: 0x%p\n", (void*)pebPtr);
            return pebPtr;
        }
    }

    // Method 3: Ask the user
    char inputAddress[32] = {0};
    if (GuiGetLineWindow("PEB Address", inputAddress))
    {
        duint userPebAddress = 0;
        if (sscanf(inputAddress, "0x%llx", (unsigned long long*)&userPebAddress) == 1 ||
            sscanf(inputAddress, "%llu", (unsigned long long*)&userPebAddress) == 1)
        {
            if (userPebAddress != 0)
            {
                _plugin_logprintf("[Scenario Scripts] Using manually entered PEB address: 0x%p\n", (void*)userPebAddress);
                return userPebAddress;
            }
        }
    }

    _plugin_logprintf("[Scenario Scripts] Could not determine PEB address\n");
    return 0;
}

// Helper to resolve TEB address
duint ScenarioManager::ResolveTebAddress()
{
    _plugin_logprintf("[Scenario Scripts] Trying to locate TEB address...\n");

    // Method 1: Try direct TEB evaluation
    bool success = false;
    duint tebAddress = DbgEval("teb", &success);
    if (success && tebAddress != 0)
    {
        _plugin_logprintf("[Scenario Scripts] Using TEB address from DbgEval: 0x%p\n", (void*)tebAddress);
        return tebAddress;
    }

    // Method 2: Try to get TEB via segment registers
    _plugin_logprintf("[Scenario Scripts] DbgEval failed, trying alternative method...\n");

    // In 64-bit, GS points to TEB
    bool gsSuccess = false;
    duint gsBase = DbgEval("@gs", &gsSuccess);
    if (gsSuccess && gsBase != 0)
    {
        _plugin_logprintf("[Scenario Scripts] Using TEB address from GS register: 0x%p\n", (void*)gsBase);
        return gsBase;
    }

    // If that fails, try FS
    bool fsSuccess = false;
    duint fsBase = DbgEval("@fs", &fsSuccess);
    if (fsSuccess && fsBase != 0)
    {
        _plugin_logprintf("[Scenario Scripts] Using TEB address from FS register: 0x%p\n", (void*)fsBase);
        return fsBase;
    }

    // Method 3: Ask the user
    char inputAddress[32] = {0};
    if (GuiGetLineWindow("TEB Address", inputAddress))
    {
        duint userTebAddress = 0;
        if (sscanf(inputAddress, "0x%llx", (unsigned long long*)&userTebAddress) == 1 ||
            sscanf(inputAddress, "%llu", (unsigned long long*)&userTebAddress) == 1)
        {
            if (userTebAddress != 0)
            {
                _plugin_logprintf("[Scenario Scripts] Using manually entered TEB address: 0x%p\n", (void*)userTebAddress);
                return userTebAddress;
            }
        }
    }

    _plugin_logprintf("[Scenario Scripts] Could not determine TEB address\n");
    return 0;
}

// Helper method to display structure members
void ScenarioManager::DisplayStructureMembers(const StructureDefinition& structDef, duint baseAddress)
{
    duint currentOffset = 0;

    // Manually extract fields for most popular structures if parsing failed
    std::vector<std::string> members;
    std::vector<std::string> types;

    if (!structDef.members.empty()) {
        members = structDef.members;
        types = structDef.memberTypes;
    }
    else {
        // Extract fields from definition for common structures
        if (structDef.name == "PROCESS_INFORMATION") {
            members = {"hProcess", "hThread", "dwProcessId", "dwThreadId"};
            types = {"HANDLE", "HANDLE", "DWORD", "DWORD"};
        }
        else if (structDef.name == "STARTUPINFO") {
            members = {"cb", "lpReserved", "lpDesktop", "lpTitle", "dwX", "dwY", "dwXSize",
                      "dwYSize", "dwXCountChars", "dwYCountChars", "dwFillAttribute", "dwFlags",
                      "wShowWindow", "cbReserved2", "lpReserved2", "hStdInput", "hStdOutput", "hStdError"};
            types = {"DWORD", "LPSTR", "LPSTR", "LPSTR", "DWORD", "DWORD", "DWORD", "DWORD",
                    "DWORD", "DWORD", "DWORD", "DWORD", "WORD", "WORD", "LPBYTE", "HANDLE", "HANDLE", "HANDLE"};
        }
        // Add other common structures as needed
    }

    _plugin_logprintf("Base address: 0x%p\n\n", (void*)baseAddress);

    // Process each field
    for (size_t i = 0; i < members.size(); i++) {
        const std::string& member = members[i];
        std::string memberType = (i < types.size()) ? types[i] : "DWORD"; // Default to DWORD

        // Determine size based on type
        size_t size = GetTypeSize(memberType);

        // Handle special cases like arrays
        if (member.find('[') != std::string::npos) {
            // This is an array, show special formatting
            _plugin_logprintf("  0x%04X %-20s : %-10s <Array>\n",
                             (unsigned int)currentOffset, member.c_str(), memberType.c_str());

            // Calculate array size and adjust offset
            std::regex arrayRegex("\\[(\\d+)\\]");
            std::smatch match;
            if (std::regex_search(member, match, arrayRegex)) {
                int arraySize = std::stoi(match[1].str());
                currentOffset += size * arraySize;
            }
            continue;
        }

        // Format: Offset MemberName : Type = Value
        _plugin_logprintf("  0x%04X %-20s : %-10s = ",
                         (unsigned int)currentOffset, member.c_str(), memberType.c_str());

        // Read memory at the current offset
        unsigned char buffer[32] = {0}; // Large enough for any standard type
        if (DbgMemRead(baseAddress + currentOffset, buffer, size)) {
            // Display value based on type
            DisplayMemberValue(buffer, memberType, baseAddress + currentOffset);
        } else {
            _plugin_logprintf("<Failed to read memory>\n");
        }

        currentOffset += size;
    }
}

// Helper to display member value
void ScenarioManager::DisplayMemberValue(const unsigned char* buffer, const std::string& typeName, duint address)
{
    // Handle different types differently
    if (typeName == "FILETIME") {
        FILETIME ft;
        memcpy(&ft, buffer, sizeof(FILETIME));
        SYSTEMTIME st;
        if (FileTimeToSystemTime(&ft, &st)) {
            _plugin_logprintf("%04d-%02d-%02d %02d:%02d:%02d.%03d\n",
                st.wYear, st.wMonth, st.wDay, st.wHour, st.wMinute, st.wSecond, st.wMilliseconds);
        } else {
            _plugin_logprintf("<Invalid FILETIME>\n");
        }
    }
    else if (typeName.find("HANDLE") != std::string::npos ||
             typeName.find("LPSTR") != std::string::npos ||
             typeName.find("LPBYTE") != std::string::npos ||
             typeName.find("LP") == 0 || typeName == "PVOID") {
        // Handle pointers
        duint ptrValue = 0;
        memcpy(&ptrValue, buffer, sizeof(void*));

        if (ptrValue == 0) {
            _plugin_logprintf("NULL\n");
        } else {
            _plugin_logprintf("0x%p", (void*)ptrValue);

            // For string pointers, try to read the string
            if (typeName.find("LPSTR") != std::string::npos && ptrValue != 0) {
                char strBuffer[256] = {0};
                if (DbgMemRead(ptrValue, strBuffer, sizeof(strBuffer) - 1)) {
                    _plugin_logprintf(" -> \"%s\"", strBuffer);
                }
            }

            _plugin_logprintf("\n");
        }
    }
    else if (typeName == "BYTE" || typeName == "CHAR") {
        unsigned char value = buffer[0];
        if (typeName == "CHAR" && isprint(value)) {
            _plugin_logprintf("0x%02X ('%c')\n", value, value);
        } else {
            _plugin_logprintf("0x%02X\n", value);
        }
    }
    else if (typeName == "WORD" || typeName == "SHORT") {
        WORD value = *(WORD*)buffer;
        _plugin_logprintf("0x%04X (%d)\n", value, (SHORT)value);
    }
    else if (typeName == "DWORD" || typeName == "LONG") {
        DWORD value = *(DWORD*)buffer;
        _plugin_logprintf("0x%08X (%d)\n", value, (int)value);
    }
    else if (typeName == "UNICODE_STRING") {
        // Special handling for UNICODE_STRING
        struct {
            WORD Length;
            WORD MaximumLength;
            PWSTR Buffer;
        } unicodeString;
        memcpy(&unicodeString, buffer, sizeof(unicodeString));

        _plugin_logprintf("Length=%d, MaxLength=%d, Buffer=0x%p",
            unicodeString.Length, unicodeString.MaximumLength, unicodeString.Buffer);

        // Try to read the string if buffer is valid
        if (unicodeString.Buffer && unicodeString.Length > 0 && unicodeString.Length < 2048) {
            wchar_t* wstr = new wchar_t[unicodeString.Length/2 + 1];
            if (DbgMemRead((duint)unicodeString.Buffer, wstr, unicodeString.Length)) {
                wstr[unicodeString.Length/2] = 0; // Null terminate

                // Convert to narrow string for logging
                char* str = new char[unicodeString.Length + 1];
                wcstombs(str, wstr, unicodeString.Length + 1);
                _plugin_logprintf(" -> \"%s\"", str);
                delete[] str;
            }
            delete[] wstr;
        }
        _plugin_logprintf("\n");
    }
    else {
        // Generic handler for other types
        duint value = 0;
        memcpy(&value, buffer, GetTypeSize(typeName));
        _plugin_logprintf("0x%llX\n", (unsigned long long)value);
    }
}

// Calculate total size of a structure
size_t ScenarioManager::CalculateStructureSize(const StructureDefinition& structDef)
{
    size_t totalSize = 0;

    // If we have member information, calculate based on that
    if (!structDef.members.empty())
    {
        for (size_t i = 0; i < structDef.members.size(); i++)
        {
            const std::string& member = structDef.members[i];
            std::string memberType = (i < structDef.memberTypes.size()) ? structDef.memberTypes[i] : "DWORD";

            // Get size for this type
            size_t size = GetTypeSize(memberType);

            // Check if it's an array
            if (member.find('[') != std::string::npos)
            {
                // Extract array size
                std::regex arrayRegex("\\[(\\d+)\\]");
                std::smatch match;
                if (std::regex_search(member, match, arrayRegex))
                {
                    int arraySize = std::stoi(match[1].str());
                    size *= arraySize;
                }
            }

            totalSize += size;
        }
    }
    else
    {
        // For structures without member info, use default sizes for known structures
        if (structDef.name == "PROCESS_INFORMATION")
        {
            totalSize = sizeof(HANDLE) * 2 + sizeof(DWORD) * 2;
        }
        else if (structDef.name == "STARTUPINFO")
        {
            totalSize = sizeof(DWORD) + sizeof(LPSTR) * 3 + sizeof(DWORD) * 8 + sizeof(WORD) * 2 +
                        sizeof(LPBYTE) + sizeof(HANDLE) * 3;
        }
        else if (structDef.name == "SECURITY_ATTRIBUTES")
        {
            totalSize = sizeof(DWORD) + sizeof(LPVOID) + sizeof(BOOL);
        }
        else
        {
            // For unknown structures, use a reasonable default
            totalSize = 64;
        }
    }

    return totalSize;
}
